/*
	SoundView.h
	Sound Kit, Release 3.0
	Copyright (c) 1988, 1989, 1990, 1991, NeXT, Inc.  All rights reserved. 
*/

#import <streams/streams.h>
#import <AppKit/NSView.h>

@interface SoundView : NSView {
    id sound;
    id _private;		/* 3.0 */
    id delegate;
    NSRect selectionRect;
    int displayMode;
    float backgroundGray;
    float foregroundGray;
    float reductionFactor;
    struct {
	unsigned int disabled:1;
	unsigned int continuous:1;
	unsigned int calcDrawInfo:1;
	unsigned int selectionDirty:1;
	unsigned int autoscale:1;
	unsigned int bezeled:1;
	unsigned int notEditable:1;
	unsigned int notOptimizedForSpeed:1;	/* 3.0 */
	unsigned int _reservedFlags:8;
    } svFlags;
    id _scratchSound;
    int _currentSample;
}

/*
 * Display modes
 */
#define NX_SOUNDVIEW_MINMAX 0
#define NX_SOUNDVIEW_WAVE 1

/*
 * OBSOLETE display modes - use the NX ones above.
 */
#define SK_DISPLAY_MINMAX 0
#define SK_DISPLAY_WAVE 1

+ initialize;
- initFrame:(NSRect)aRect;
- free;
- write:(NXTypedStream *) stream;
- read:(NXTypedStream *) stream;
- (BOOL)writeSelectionToPasteboard:pboard types:(NXAtom *)types;
- readSelectionFromPasteboard:pboard;
- validRequestorForSendType:(NSString *)sendType andReturnType:(NSString *)returnType;
- pasteboard:pb provideData:(const char *)type;
- sound;
- setSound:aSound;
- setReductionFactor:(float)reductionFactor;
- (float)reductionFactor;
- delegate;
- setDelegate:anObject;
- tellDelegate:(SEL)theMessage;
- getSelection:(int *)firstSample size:(int *)sampleCount;
- setSelection:(int)firstSample size:(int)sampleCount;
- hideCursor;
- showCursor;
- setBackgroundGray:(float)aGray;
- (float)backgroundGray;
- setForegroundGray:(float)aGray;
- (float)foregroundGray;
- (int)displayMode;
- setDisplayMode:(int)aMode;
- (BOOL)isContinuous;
- setContinuous:(BOOL)aFlag;
- (BOOL)isEnabled;
- setEnabled:(BOOL)aFlag;
- (BOOL)isEditable;
- setEditable:(BOOL)aFlag;
- (BOOL)isPlayable;
- (BOOL)isBezeled;
- setBezeled:(BOOL)aFlag;
- (BOOL)isAutoScale;
- setAutoscale:(BOOL)aFlag;
- (BOOL)isOptimizedForSpeed;		/* 3.0 */
- setOptimizedForSpeed:(BOOL)aFlag;	/* 3.0 */
- scaleToFit;
- sizeToFit;
- drawSamplesFrom:(int)startX to:(int)endX;	/* 3.0 */
- drawSelf:(NSRect)rect;
- mouseDown:(NSEvent *)theEvent;
- (BOOL)acceptsFirstResponder;
- becomeFirstResponder; 
- resignFirstResponder; 
- selectAll:sender;
- delete:sender;
- cut:sender;
- copy:sender;
- paste:sender;
- play:sender;
- record:sender;
- stop:sender;
- pause:sender;
- resume:sender;
- soundBeingProcessed;
- willPlay:sender;
- didPlay:sender;
- willRecord:sender;
- didRecord:sender;
- hadError:sender;

@end

@interface SoundViewDelegate : Object
- soundDidChange:sender;
- selectionDidChange:sender;
- willRecord:sender;
- didRecord:sender;
- willPlay:sender;
- didPlay:sender;
- hadError:sender;
- willFree:sender;
@end

